@@ -22,13 +22,16 @@ public:
 signals:
     void locationChanged(QString dir);
 
+protected:
+    void paintEvent(QPaintEvent *);
+
 private slots:
     void chooseFolder();
     void systemDirChosen();
     void iTunesDirChosen();
 
 private:
-    QBoxLayout *welcomeLayout;
+    QLabel *welcomeLabel;
     QLabel *tipLabel;
     QPushButton *cancelButton;
 
