@@ -20,9 +20,10 @@
 #ifdef Q_WS_X11
 #include "gnomeglobalshortcutbackend.h"
 #endif
-#ifdef APP_MAC
+#ifdef QT_MAC_USE_COCOA
 #include "local/mac/mac_startup.h"
 #endif
+#include "collectionsuggester.h"
 
 /*
 class CentralWidget : public QWidget {
@@ -50,12 +51,12 @@ MainWindow::MainWindow() {
     aboutView = 0;
     contextualView = 0;
 
-    /*
     toolbarSearch = new SearchLineEdit(this);
     toolbarSearch->setFont(qApp->font());
     toolbarSearch->setMinimumWidth(toolbarSearch->fontInfo().pixelSize()*15);
-    // TODO connect(toolbarSearch, SIGNAL(search(const QString&)), searchView, SLOT(watch(const QString&)));
-    */
+    toolbarSearch->setSuggester(new CollectionSuggester(this));
+    connect(toolbarSearch, SIGNAL(search(const QString&)), SLOT(search(const QString&)));
+    connect(toolbarSearch, SIGNAL(cleared()), SLOT(searchCleared()));
 
     // build ui
     createActions();
@@ -72,11 +73,15 @@ MainWindow::MainWindow() {
 
     setCentralWidget(views);
 
+    // restore window position
+    readSettings();
+
     // show the initial view
     Database &db = Database::instance();
     if (db.status() == ScanComplete) {
 
         showMediaView();
+        loadPlaylist();
 
         // update the collection when idle
         QTimer::singleShot(1000, this, SLOT(startIncrementalScan()));
@@ -92,16 +97,13 @@ MainWindow::MainWindow() {
     // event filter to block ugly toolbar tooltips
     qApp->installEventFilter(this);
 
-    // restore window position
-    readSettings();
-
     // Global shortcuts
     GlobalShortcuts &shortcuts = GlobalShortcuts::instance();
 #ifdef Q_WS_X11
     if (GnomeGlobalShortcutBackend::IsGsdAvailable())
         shortcuts.setBackend(new GnomeGlobalShortcutBackend(&shortcuts));
 #endif
-#ifdef APP_MAC
+#ifdef QT_MAC_USE_COCOA
     mac::MacSetup();
 #endif
     connect(&shortcuts, SIGNAL(PlayPause()), playAct, SLOT(trigger()));
@@ -166,8 +168,8 @@ void MainWindow::createActions() {
     */
 
     skipBackwardAct = new QAction(
-            QtIconLoader::icon("media-skip-backward"),
-            tr("P&revious"), this);
+                QtIconLoader::icon("media-skip-backward"),
+                tr("P&revious"), this);
     skipBackwardAct->setStatusTip(tr("Go back to the previous track"));
     skipBackwardAct->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Left));
 #if QT_VERSION >= 0x040600
@@ -177,8 +179,8 @@ void MainWindow::createActions() {
     actions->insert("previous", skipBackwardAct);
 
     skipForwardAct = new QAction(
-            QtIconLoader::icon("media-skip-forward"),
-            tr("&Next"), this);
+                QtIconLoader::icon("media-skip-forward"),
+                tr("&Next"), this);
     skipForwardAct->setStatusTip(tr("Skip to the next track"));
     skipForwardAct->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Right));
 #if QT_VERSION >= 0x040600
@@ -188,8 +190,8 @@ void MainWindow::createActions() {
     actions->insert("skip", skipForwardAct);
 
     playAct = new QAction(
-            QtIconLoader::icon("media-playback-start"),
-            tr("&Play"), this);
+                QtIconLoader::icon("media-playback-start"),
+                tr("&Play"), this);
     playAct->setStatusTip(tr("Start playback"));
     playAct->setShortcuts(QList<QKeySequence>()
                           << QKeySequence(Qt::Key_Space)
@@ -201,10 +203,10 @@ void MainWindow::createActions() {
 #endif
     actions->insert("play", playAct);
 
-#ifndef APP_MAC
+#ifndef APP_MAC_NO
     fullscreenAct = new QAction(
-            QtIconLoader::icon("view-restore"),
-            tr("&Full Screen"), this);
+                QtIconLoader::icon("view-restore"),
+                tr("&Full Screen"), this);
     fullscreenAct->setStatusTip(tr("Go full screen"));
     fullscreenAct->setShortcut(QKeySequence(Qt::ALT + Qt::Key_Return));
     fullscreenAct->setShortcuts(QList<QKeySequence>()
@@ -273,7 +275,7 @@ void MainWindow::createActions() {
     actions->insert("clearPlaylist", action);
 
     action = new QAction(
-            QtIconLoader::icon("media-playlist-shuffle"), tr("&Shuffle"), this);
+                QtIconLoader::icon("media-playlist-shuffle"), tr("&Shuffle"), this);
     action->setStatusTip(tr("Random playlist mode"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_S));
     action->setCheckable(true);
@@ -281,8 +283,8 @@ void MainWindow::createActions() {
     actions->insert("shufflePlaylist", action);
 
     action = new QAction(
-            QtIconLoader::icon("media-playlist-repeat"),
-            tr("&Repeat"), this);
+                QtIconLoader::icon("media-playlist-repeat"),
+                tr("&Repeat"), this);
     action->setStatusTip(tr("Play first song again after all songs are played"));
     action->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
     action->setCheckable(true);
@@ -338,7 +340,7 @@ void MainWindow::createActions() {
         // action->setToolTip(action->statusTip());
 
         // make the actions work when in fullscreen
-        // action->setShortcutContext(Qt::ApplicationShortcut);
+        action->setShortcutContext(Qt::ApplicationShortcut);
 
         // show keyboard shortcuts in the status bar
         if (!action->shortcut().isEmpty())
@@ -356,7 +358,6 @@ void MainWindow::createMenus() {
 
     QMap<QString, QMenu*> *menus = The::globalMenus();
 
-
     fileMenu = menuBar()->addMenu(tr("&Application"));
     fileMenu->addAction(chooseFolderAct);
 #ifndef APP_MAC
@@ -388,7 +389,7 @@ void MainWindow::createMenus() {
 
     QMenu *viewMenu = menuBar()->addMenu(tr("&View"));
     viewMenu->addAction(contextualAct);
-#ifndef APP_MAC
+#ifndef APP_MAC_NO
     viewMenu->addSeparator();
     viewMenu->addAction(fullscreenAct);
 #endif
@@ -412,15 +413,10 @@ void MainWindow::createToolBars() {
 #endif
     mainToolBar->setFloatable(false);
     mainToolBar->setMovable(false);
-#ifdef APP_MAC
+
     mainToolBar->addAction(skipBackwardAct);
     mainToolBar->addAction(playAct);
     mainToolBar->addAction(skipForwardAct);
-#else
-    mainToolBar->addAction(playAct);
-    mainToolBar->addAction(skipBackwardAct);
-    mainToolBar->addAction(skipForwardAct);
-#endif
 
     mainToolBar->addAction(contextualAct);
 
@@ -456,8 +452,8 @@ void MainWindow::createToolBars() {
     QSlider* volumeQSlider = volumeSlider->findChild<QSlider*>();
     if (volumeQSlider)
         volumeQSlider->setStatusTip(tr("Press %1 to raise the volume, %2 to lower it").arg(
-                volumeUpAct->shortcut().toString(QKeySequence::NativeText),
-                volumeDownAct->shortcut().toString(QKeySequence::NativeText)));
+                                        volumeUpAct->shortcut().toString(QKeySequence::NativeText),
+                                        volumeDownAct->shortcut().toString(QKeySequence::NativeText)));
 
     // this makes the volume slider smaller
     volumeSlider->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
@@ -465,7 +461,7 @@ void MainWindow::createToolBars() {
 
     mainToolBar->addWidget(new Spacer());
 
-    // mainToolBar->addWidget(new Spacer(mainToolBar, toolbarSearch));
+    mainToolBar->addWidget(toolbarSearch);
 
     addToolBar(mainToolBar);
 }
@@ -474,7 +470,7 @@ void MainWindow::createStatusBar() {
 
     // remove ugly borders on OSX
     statusBar()->setStyleSheet(
-            "::item{border:0 solid} QStatusBar, QToolBar {padding:0;margin:0} QToolButton {padding:1px}");
+                "::item{border:0 solid} QStatusBar, QToolBar {padding:0;margin:0} QToolButton {padding:1px}");
 
     statusToolBar = new QToolBar(this);
     statusToolBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
@@ -497,6 +493,14 @@ void MainWindow::createStatusBar() {
 void MainWindow::readSettings() {
     QSettings settings;
     restoreGeometry(settings.value("geometry").toByteArray());
+#ifdef APP_MAC
+    if (!isMaximized())
+#ifdef QT_MAC_USE_COCOA
+        move(x(), y() + 10);
+#else
+        move(x(), y() + mainToolBar->height() + 8);
+#endif
+#endif
     m_maximized = isMaximized();
     The::globalActions()->value("shufflePlaylist")->setChecked(settings.value("shuffle").toBool());
     The::globalActions()->value("repeatPlaylist")->setChecked(settings.value("repeat").toBool());
@@ -621,6 +625,7 @@ void MainWindow::donate() {
 }
 
 void MainWindow::quit() {
+    savePlaylist();
     writeSettings();
     /*
     CollectionScanner *scanner = CollectionScanner::instance();
@@ -654,6 +659,11 @@ void MainWindow::showMediaView() {
         connect(playAct, SIGNAL(triggered()), mediaView, SLOT(playPause()));
         views->addWidget(mediaView);
     }
+
+    if (views->currentWidget() == contextualView) {
+        hideContextualView();
+    }
+
     mediaView->setFocus();
     showView(mediaView);
 }
@@ -664,14 +674,7 @@ void MainWindow::toggleContextualView() {
         views->addWidget(contextualView);
     }
     if (views->currentWidget() == contextualView) {
-        goBack();
-        contextualAct->setChecked(false);
-
-        QList<QKeySequence> shortcuts;
-        shortcuts << QKeySequence(Qt::CTRL + Qt::Key_Return);
-        contextualAct->setShortcuts(shortcuts);
-        // stopAct->setShortcut(QKeySequence(Qt::Key_Escape));
-
+        hideContextualView();
     } else {
         Track *track = mediaView->getActiveTrack();
         if (track) {
@@ -688,6 +691,16 @@ void MainWindow::toggleContextualView() {
     }
 }
 
+void MainWindow::hideContextualView() {
+    goBack();
+    contextualAct->setChecked(false);
+
+    QList<QKeySequence> shortcuts;
+    shortcuts << QKeySequence(Qt::CTRL + Qt::Key_Return);
+    contextualAct->setShortcuts(shortcuts);
+    // stopAct->setShortcut(QKeySequence(Qt::Key_Escape));
+}
+
 void MainWindow::updateContextualView(Track *track) {
     if (views->currentWidget() == contextualView) {
         contextualView->setTrack(track);
@@ -706,6 +719,7 @@ void MainWindow::startFullScan(QString directory) {
     CollectionScannerThread *scannerThread = new CollectionScannerThread();
     collectionScannerView->setCollectionScannerThread(scannerThread);
     scannerThread->setDirectory(directory);
+    connect(scannerThread, SIGNAL(finished()), SLOT(fullScanFinished()));
     scannerThread->start();
 
     if (mediaView) {
@@ -717,6 +731,10 @@ void MainWindow::startFullScan(QString directory) {
     }
 }
 
+void MainWindow::fullScanFinished() {
+    QApplication::alert(this, 0);
+}
+
 void MainWindow::startIncrementalScan() {
     statusBar()->showMessage(tr("Updating collection..."));
     chooseFolderAct->setEnabled(false);
@@ -758,28 +776,28 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
         }
         break;
 
-         case Phonon::PlayingState:
+    case Phonon::PlayingState:
         // stopAct->setEnabled(true);
         contextualAct->setEnabled(true);
         break;
 
-         case Phonon::StoppedState:
+    case Phonon::StoppedState:
         // stopAct->setEnabled(false);
         contextualAct->setEnabled(false);
         break;
 
-         case Phonon::PausedState:
+    case Phonon::PausedState:
         // stopAct->setEnabled(true);
         break;
 
-         case Phonon::BufferingState:
-         case Phonon::LoadingState:
+    case Phonon::BufferingState:
+    case Phonon::LoadingState:
         // stopAct->setEnabled(true);
         currentTime->clear();
         totalTime->clear();
         break;
 
-         default:
+    default:
         contextualAct->setEnabled(false);
     }
 }
@@ -803,14 +821,16 @@ void MainWindow::toggleFullscreen() {
 
         mainToolBar->removeAction(fullscreenAct);
 
-        seekSlider->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
-        mainToolBar->updateGeometry();
-
-        setUnifiedTitleAndToolBarOnMac(true);
-        showNormal();
+#if APP_MAC
+        setCentralWidget(views);
+        views->showNormal();
+        show();
+        mediaView->setFocus();
+#else
+        // mainToolBar->show();
         if (m_maximized) showMaximized();
-
-        // QTimer::singleShot(100, this, SLOT(enableUnifiedToolbar()));
+        else showNormal();
+#endif
 
         activateWindow();
 
@@ -829,8 +849,15 @@ void MainWindow::toggleFullscreen() {
 
         mainToolBar->addAction(fullscreenAct);
 
-        setUnifiedTitleAndToolBarOnMac(false);
+#ifdef APP_MAC
+        hide();
+        views->setParent(0);
+        QTimer::singleShot(0, views, SLOT(showFullScreen()));
+#else
+        // mainToolBar->hide();
         showFullScreen();
+#endif
+
     }
 
 #ifndef APP_MAC
@@ -843,10 +870,8 @@ void MainWindow::toggleFullscreen() {
 }
 
 void MainWindow::searchFocus() {
-    QWidget *view = views->currentWidget();
-    if (view == mediaView) {
-        // toolbarSearch->setFocus();
-    }
+    toolbarSearch->selectAll();
+    toolbarSearch->setFocus();
 }
 
 void MainWindow::initPhonon() {
@@ -974,10 +999,14 @@ void MainWindow::checkForUpdate() {
 }
 
 void MainWindow::gotNewVersion(QString version) {
+#ifdef APP_MAC_STORE
+    return;
+#endif
+
     QLabel *message = new QLabel(this);
 
     QString text = tr("%1 %2 is available!").arg(
-            Constants::APP_NAME, version);
+                Constants::APP_NAME, version);
 
 #if !defined(APP_MAC) && !defined(Q_WS_WIN)
     text += " " + tr("Please <a href='%2'>update now</a>.")
@@ -988,3 +1017,55 @@ void MainWindow::gotNewVersion(QString version) {
     if (updateChecker) delete updateChecker;
     statusBar()->addWidget(message);
 }
+
+QString MainWindow::playlistPath() {
+    const QString storageLocation =
+            QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+    // We need to use a default name, so why not the application one? (minitunes.pls sounds fine)
+    return QString("%1/%2.pls").arg(storageLocation).arg(qApp->applicationName().toLower());
+}
+
+void MainWindow::savePlaylist() {
+    const PlaylistModel* playlistModel = mediaView->getPlaylistModel();
+    if (playlistModel == 0)
+        return;
+
+    QString plsPath = playlistPath();
+
+    QFile plsFile(plsPath);
+    QTextStream plsStream(&plsFile);
+    if (!plsFile.open(QFile::WriteOnly | QFile::Truncate)) {
+        qDebug() << "Cannot open file" << plsPath;
+        return;
+    }
+    if (!playlistModel->saveTo(plsStream)) {
+        qDebug() << "Error saving playlist";
+    }
+}
+
+void MainWindow::loadPlaylist() {
+
+    QString plsPath = playlistPath();
+    if (!QFile::exists(plsPath))
+        return;
+    PlaylistModel* playlistModel = mediaView->getPlaylistModel();
+    if (playlistModel == 0)
+        return;
+    // qDebug() << "Loading playlist: " << plsPath;
+    QFile plsFile(plsPath);
+    QTextStream plsStream(&plsFile);
+    if ( plsFile.open(QFile::ReadOnly) )
+        playlistModel->loadFrom(plsStream);
+    else
+        qDebug() << "Cannot open file" << plsPath;
+}
+
+void MainWindow::search(QString query) {
+    showMediaView();
+    mediaView->search(query);
+    toolbarSearch->setFocus();
+}
+
+void MainWindow::searchCleared() {
+    mediaView->search("");
+}
